package cc.winboll.studio.libappbase.utils;

/**
 * @Author ZhanGSKen<zhangsken@188.com>
 * @Date 2025/06/04 17:21
 * @Describe 应用登录与接口工具
 */
import android.content.Context;
import cc.winboll.studio.libappbase.LogUtils;
import cc.winboll.studio.libappbase.models.APPModel;
import cc.winboll.studio.libappbase.models.UserInfoModel;
import java.io.File;
import java.security.KeyPair;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.nio.charset.StandardCharsets;

public class YunUtils {
    public static final String TAG = "YunUtils";
    // 私有静态实例，类加载时创建
    private static volatile YunUtils INSTANCE;
    Context mContext;
    UserInfoModel mUserInfoModel;
    String token = "";
    String mDataFolderPath = "";
    String mUserInfoModelPath = "";

    // 私有构造方法，防止外部实例化
    private YunUtils(Context context) {
        LogUtils.d(TAG, "YunUtils");
        mContext = context;
        mDataFolderPath = mContext.getExternalFilesDir(TAG).toString();
        File fTest = new File(mDataFolderPath);
        if (!fTest.exists()) {
            fTest.mkdirs();
        }
        mUserInfoModelPath = mDataFolderPath + File.separator + "UserInfoModel.rsajson";

    }

    // 公共静态方法，返回唯一实例
    public static synchronized YunUtils getInstance(Context context) {
        LogUtils.d(TAG, "getInstance");
        if (INSTANCE == null) {
            INSTANCE = new YunUtils(context);
        }
        return INSTANCE;
    }

    public void login() {
        String token = getHostToken(null);
        LogUtils.d(TAG, String.format("login token is %s", token));
        saveLocalToken(token);
    }

    public void logout() {

    }

    public void checkLoginStatus() {
        String token = getLocalToken();
        LogUtils.d(TAG, String.format("checkLoginStatus token is %s", token));
    }

    String getLocalToken() {
        UserInfoModel userInfoModel = loadUserInfoModel();
        return (userInfoModel == null) ?"": userInfoModel.getToken();
    }

    String getHostToken(UserInfoModel userInfoModel) {
        String token = "uuyyhh";
        return token;
    }

    void saveLocalToken(String token) {
        UserInfoModel userInfoModel = new UserInfoModel();
        userInfoModel.setToken(token);
        saveUserInfoModel(userInfoModel);
    }

    UserInfoModel loadUserInfoModel() {
        LogUtils.d(TAG, "loadUserInfoModel");
        if (new File(mUserInfoModelPath).exists()) {
            try {
                // 加载加密后的模型数据
                byte[] encryptedData = FileUtils.readFileToByteArray(mUserInfoModelPath);
                // 加载 RSA 工具
                RSAUtils utils = RSAUtils.getInstance(mContext);
                KeyPair keyPair = utils.getOrGenerateKeys();
                //PublicKey publicKey = keyPair.getPublic();
                PrivateKey privateKey = keyPair.getPrivate();
                // 私钥解密模型数据
                String szInfo = utils.decryptWithPrivateKey(encryptedData, keyPair.getPrivate());
                LogUtils.d(TAG, String.format("szInfo %s", szInfo));
                mUserInfoModel = UserInfoModel.parseStringToBean(szInfo, UserInfoModel.class);
                if (mUserInfoModel == null) {
                    LogUtils.d(TAG, "模型数据解析为空数据。");
                }
                LogUtils.d(TAG, "UserInfoModel 解密加载结束。");
            } catch (Exception e) {
                LogUtils.d(TAG, e, Thread.currentThread().getStackTrace());
            }
        } else {
            LogUtils.d(TAG, "云服务登录信息不存在。");
            mUserInfoModel = null;
        }
        return mUserInfoModel;
    }

    void saveUserInfoModel(UserInfoModel userInfoModel) {
        LogUtils.d(TAG, "saveUserInfoModel");
        try {
            String szInfo = userInfoModel.toString();
            LogUtils.d(TAG, "原始数据: " + szInfo);

            RSAUtils utils = RSAUtils.getInstance(mContext);
            KeyPair keyPair = utils.getOrGenerateKeys();
            PublicKey publicKey = keyPair.getPublic();

            // 公钥加密（传入字节数组，避免中间字符串转换）
            byte[] encryptedData = utils.encryptWithPublicKey(szInfo, publicKey);

            // 保存加密字节数组到文件（直接操作字节，无需转字符串）
            FileUtils.writeByteArrayToFile(encryptedData, mUserInfoModelPath);
            LogUtils.d(TAG, "加密数据已保存");

            // 测试解密（仅调试用）
            String szInfo2 = utils.decryptWithPrivateKey(encryptedData, keyPair.getPrivate());
            LogUtils.d(TAG, "解密结果: " + szInfo2);

            mUserInfoModel = UserInfoModel.parseStringToBean(szInfo2, UserInfoModel.class);
            if (mUserInfoModel == null) {
                LogUtils.d(TAG, "模型解析失败");
            }
        } catch (Exception e) {
            LogUtils.d(TAG, "加密/解密失败: " + e.getMessage());
        }
    }
}
